# -*- coding: utf-8 -*-
import pytest

from pyettj import get_ettj, plot_ettj
data = "2021/05/18"

class TestClass():

    def test_ettj(self):
        self.ettj_dataframe = get_ettj(data)
        self.curvas = self.ettj_dataframe.columns.tolist()[1:]
        self.curva = self.curvas[2] #selic
        return self.ettj_dataframe, self.curva

    def test_plot_ettj(self):
        self.ettj_dataframe, self.curva = self.test_ettj()
        plot_ettj(self.ettj_dataframe.drop(self.ettj_dataframe.columns[0],axis=1), self.curva, data)

    def test_raises(self):
        with pytest.raises(Exception) as error1:
            get_ettj(18)
        assert str(error1.value) == "O parametro data deve ser em formato string, exemplo: '18/05/2021'"
        with pytest.raises(Exception) as error2:
            get_ettj("teste")
        assert str(error2.value) == "O parametro data deve ser em formato string, exemplo: '18/05/2021'"  

TestClass()


# -*- coding: utf-8 -*-
import pandas as pd # type: ignore

pd.set_option('display.float_format', lambda x: '%.5f' % x)
pd.set_option('display.max_rows',100)
pd.set_option('display.max_columns',10)
pd.set_option('display.width',1000)

def get_fourth_table(main_table):
    count=0
    dias_corridos, iene, spread_libor_eur_usd, libor = [], [], [], []
    table_names, sub_table_names = [], []
    for i,row in enumerate(main_table.find_all('td')):
        if i <= 3:
            table_names.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
        elif i >= 4 and i <= 6:
             sub_table_names.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
        else:     
            count+=1
            if count==1: dias_corridos.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==2: iene.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==3: spread_libor_eur_usd.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==4: 
                libor.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
                count=0
            
    all_data = [dias_corridos, iene, spread_libor_eur_usd, libor]
    final_table = pd.DataFrame(columns=table_names)
    for i, col in enumerate(final_table.columns.tolist()):
        if i==0:
            final_table[col] = all_data[i]
            final_table[col] = final_table[col].astype(int)
        else:
            final_table[col] = all_data[i]
            final_table[col] = pd.to_numeric(final_table[col].str.replace(",","."))
    return final_table

def get_third_table(main_table):
    count=0
    dias_corridos, usd, ibrx50 = [], [], []
    ibov, taxa_di_igpm, taxa_di_ipca = [], [], []
    taxa_ajuste_pre_252, taxa_ajuste_pre_360, taxa_ajuste_cupom = [], [], []
    table_names, sub_table_names = [], []
    for i,row in enumerate(main_table.find_all('td')):
        if i <= 7:
            table_names.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
        elif i >= 8 and i <= 15:
             sub_table_names.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
        else:     
            count+=1
            if count==1: dias_corridos.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==2: usd.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==3: ibrx50.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==4: ibov.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==5: taxa_di_igpm.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==6: taxa_di_ipca.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==7: taxa_ajuste_pre_252.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==8: taxa_ajuste_pre_360.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==9: 
                taxa_ajuste_cupom.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
                count=0
                
    new_cols, i = [table_names[0]], 0
    for col in table_names[1:]:
        if 'ajuste pré' in col.lower():
            new_cols.append( col+' '+sub_table_names[i])
            new_cols.append( col+' '+sub_table_names[i+1])
            i+=2
        else:
            new_cols.append( col+' '+sub_table_names[i])
            i+=1
            
    all_data = [dias_corridos, usd, ibrx50, ibov, taxa_di_igpm,
                taxa_di_ipca, taxa_ajuste_pre_252, taxa_ajuste_pre_360, taxa_ajuste_cupom]
    final_table = pd.DataFrame(columns=new_cols)
    for i, col in enumerate(final_table.columns.tolist()):
        if i==0:
            final_table[col] = all_data[i]
            final_table[col] = final_table[col].astype(int)
        else:
            final_table[col] = all_data[i]
            final_table[col] = pd.to_numeric(final_table[col].str.replace(",","."))
    return final_table

def get_second_table(main_table):
    count=0
    dias_corridos, taxa_di_eur, taxa_252_tbf_pre = [], [], []
    taxa_360_tbf_pre, taxa_252_tr_pre, taxa_360_tr_pre = [], [], []
    taxa_dolar_di, taxa_cupom_cambial, taxa_cupom_limpo = [], [], []
    table_names, sub_table_names = [], []
    for i,row in enumerate(main_table.find_all('td')):
        if i <= 6:
            table_names.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
        elif i >= 7 and i <= 14:
             sub_table_names.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
        else:     
            count+=1
            if count==1: dias_corridos.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==2: taxa_di_eur.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==3: taxa_252_tbf_pre.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==4: taxa_360_tbf_pre.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==5: taxa_252_tr_pre.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==6: taxa_360_tr_pre.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==7: taxa_dolar_di.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==8: taxa_cupom_cambial.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==9: 
                taxa_cupom_limpo.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
                count=0
                
    new_cols, i = [table_names[0]], 0
    for col in table_names[1:]:
        if 'eur' in col.lower(): new_cols.append( col+' '+sub_table_names[i])
        elif 'tbf' in col.lower():
            new_cols.append( col+' '+sub_table_names[i])
            new_cols.append( col+' '+sub_table_names[i+1])
            i+=2
        elif 'tr' in col.lower():
            new_cols.append( col+' '+sub_table_names[i])
            new_cols.append( col+' '+sub_table_names[i+1])
            i+=2
        else:
            i+=1
            new_cols.append( col+' '+sub_table_names[i])
            
    all_data = [dias_corridos, taxa_di_eur, taxa_252_tbf_pre, taxa_360_tbf_pre, taxa_252_tr_pre,
                taxa_360_tr_pre, taxa_dolar_di, taxa_cupom_cambial, taxa_cupom_limpo]
    final_table = pd.DataFrame(columns=new_cols)
    for i, col in enumerate(final_table.columns.tolist()):
        if i==0:
            final_table[col] = all_data[i]
            final_table[col] = final_table[col].astype(int)
        else:
            final_table[col] = all_data[i]
            final_table[col] = pd.to_numeric(final_table[col].str.replace(",","."))
    return final_table

def get_first_table(main_table):
    count=0
    dias_corridos, taxa_252_pre_di, taxa_360_pre_di = [], [], []
    taxa_selic_pre, taxa_252_di_tr, taxa_360_di_tr = [], [], []
    taxa_252_dolar_pre, taxa_360_dolar_pre, eur = [], [], []
    table_names, sub_table_names = [], []
    for i,row in enumerate(main_table.find_all('td')):
        if i <= 5:
            table_names.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
        elif i >= 6 and i <= 13:
             sub_table_names.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
        else:     
            count+=1
            if count==1: dias_corridos.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==2: taxa_252_pre_di.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==3: taxa_360_pre_di.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==4: taxa_selic_pre.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==5: taxa_252_di_tr.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==6: taxa_360_di_tr.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==7: taxa_252_dolar_pre.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==8: taxa_360_dolar_pre.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
            elif count==9: 
                eur.append(row.text.strip().replace('\r',' ').replace('\n',' ').replace('  ',' '))
                count=0

    new_cols, i = [table_names[0]], 0
    for col in table_names[1:]:
        if 'di x pré' in col.lower():
            new_cols.append( col+' '+sub_table_names[i])
            new_cols.append( col+' '+sub_table_names[i+1])
            i+=2
        elif 'selic' in col.lower(): new_cols.append( col+' '+sub_table_names[i])
        elif 'tr' in col.lower():
            new_cols.append( col+' '+sub_table_names[i])
            new_cols.append( col+' '+sub_table_names[i+1])
            i+=2
        elif 'dólar' in col.lower():
            new_cols.append( col+' '+sub_table_names[i])
            new_cols.append( col+' '+sub_table_names[i+1])
            i+=2
        else:
            i+=1
            new_cols.append( col+' '+sub_table_names[i])
            
    all_data = [dias_corridos, taxa_252_pre_di, taxa_360_pre_di, taxa_selic_pre, taxa_252_di_tr,
                taxa_360_di_tr, taxa_252_dolar_pre, taxa_360_dolar_pre, eur]
    final_table = pd.DataFrame(columns=new_cols)
    for i, col in enumerate(final_table.columns.tolist()):
        if i==0:
            final_table[col] = all_data[i]
            final_table[col] = final_table[col].astype(int)
        else:
            final_table[col] = all_data[i]
            final_table[col] = pd.to_numeric(final_table[col].str.replace(",","."))
    return final_table


# -*- coding: utf-8 -*-
from bs4 import BeautifulSoup # type: ignore
import requests, time
import pandas as pd # type: ignore
import matplotlib.pyplot as plt; plt.style.use('fivethirtyeight') # type: ignore
from pyettj import gettables

pd.set_option('display.float_format', lambda x: '%.5f' % x)
pd.set_option('display.max_rows',100)
pd.set_option('display.max_columns',10)
pd.set_option('display.width',1000)

def _treat_parameters(data):
    '''Checking all parameters and access to web data'''
    if isinstance(data, str)==False:
        raise ValueError("O parametro data deve ser em formato string, exemplo: '18/05/2021'")
    elif isinstance(data, str):
        try:
            data = pd.to_datetime(data).strftime("%d/%m/%Y")
            return data
        except:
            raise ValueError("O parametro data deve ser em formato string, exemplo: '18/05/2021'")   

def get_ettj(data):
    start = time.time()

    data = _treat_parameters(data)

    #data = pd.to_datetime(data).strftime("%d/%m/%Y")
    curva = "TODOS"
    url = "http://www2.bmf.com.br/pages/portal/bmfbovespa/boletim1/TxRef1.asp?Data={}&Data1=20060201&slcTaxa={}".format(data,curva)

    try:
        page = requests.get(url)
        pagetext = page.text
    except:
        raise Exception("Não foi possível conectar ao website. Tente novamente mais tarde.")

    soup = BeautifulSoup(pagetext, 'lxml')
    table1 = soup.find_all('table')[1]
    table2 = soup.find_all('table')[2]
    table3 = soup.find_all('table')[3]
    table4 = soup.find_all('table')[4]

    pandas_table1 = gettables.get_first_table(table1)
    pandas_table2 = gettables.get_second_table(table2)
    pandas_table3 = gettables.get_third_table(table3)
    pandas_table4 = gettables.get_fourth_table(table4)

    final_table_pandas = pd.concat([pandas_table1, pandas_table2, pandas_table3, pandas_table4], axis=1)
    final_table_pandas["Data"] = data
    final_table_pandas = final_table_pandas.loc[:,~final_table_pandas.columns.duplicated()]
    print("Curvas capturadas em {} segundos.".format(round(time.time()-start,2)))
    return final_table_pandas

def plot_ettj(ettj, curva, data): #pragma: no cover
    ettj_ = ettj.copy()
    data = pd.to_datetime(data).strftime("%d/%m/%Y")
    ettj_ = ettj_[ettj_.Data==data]
    ettj_.index = ettj_[ettj_.columns[0]]
    ettj_ = ettj_[[curva]]
    ettj_.plot()
    plt.title('Curva - '+curva)
    plt.xticks(rotation=45)
    plt.xlabel('Maturidade (dias)')
    plt.ylabel('Taxa (%)  ',rotation=0, labelpad=50)
    plt.tight_layout()
    plt.legend('')
    plt.show()